# -*- coding: utf-8 -*-
"""
DataLab Simple Client Widgets
-----------------------------

This module requires `guidata`_ and `qtpy`_ packages.

.. _guidata: https://pypi.org/project/guidata/
.. _qtpy: https://pypi.org/project/QtPy/

"""

# pylint: disable=unused-import
from cdlclient.widgets.connection import ConnectionDialog  # noqa: F401
from cdlclient.widgets.objectdialog import GetObjectDialog  # noqa: F401
